@@ -231,7 +231,7 @@ class APIClient(json.JSONEncoder):
         return _validate(resp).json()
 
     def revoke_token(self):
-        resp = self.session.post(self.revoke_url)
+        resp = requests.post(self.revoke_url, auth=(self.access_token, None))
         _validate(resp)
         self.auth_token = None
         self.access_token = None
@@ -287,7 +287,7 @@ class APIClient(json.JSONEncoder):
             "start_time": start_time,
             "end_time": end_time,
         }
-        resp = self._request(HttpMethod.POST, url, json=data)
+        resp = self._request(HttpMethod.POST, url, json=data, cls=Calendar)
         _validate(resp)
         return resp.json()
 
@@ -316,6 +316,7 @@ class APIClient(json.JSONEncoder):
         interval,
         start_at,
         end_at,
+        event_collection_id=None,
         buffer=None,
         round_robin=None,
         free_busy=None,
@@ -356,8 +357,10 @@ class APIClient(json.JSONEncoder):
             data["buffer"] = buffer
         if round_robin is not None:
             data["round_robin"] = round_robin
+        if event_collection_id is not None:
+            data["event_collection_id"] = event_collection_id
 
-        resp = self._request(HttpMethod.POST, url, json=data)
+        resp = self._request(HttpMethod.POST, url, json=data, cls=Calendar)
         _validate(resp)
         return resp.json()
 
@@ -410,7 +413,7 @@ class APIClient(json.JSONEncoder):
         if buffer is not None:
             data["buffer"] = buffer
 
-        resp = self._request(HttpMethod.POST, url, json=data)
+        resp = self._request(HttpMethod.POST, url, json=data, cls=Calendar)
         _validate(resp)
         return resp.json()
 
