@@ -1023,7 +1023,7 @@ def mock_message_search_response(mocked_responses, api_url):
 
     mocked_responses.add(
         responses.GET,
-        api_url + "/messages/search?q=Pinot",
+        re.compile(api_url + "/messages/search\?q=Pinot.*"),
         body=response_body,
         status=200,
         content_type="application/json",
@@ -1623,6 +1623,106 @@ def mock_components(mocked_responses, api_url):
     )
 
 
+@pytest.fixture
+def mock_create_webhook(mocked_responses, api_url, client_id):
+    webhook = {"application_id": "application-id", "id": "webhook-id", "version": "1.0"}
+
+    def callback(request):
+        try:
+            payload = json.loads(request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        if (
+            "callback_url" not in payload
+            and ("triggers" not in payload and type(payload["triggers"]) is not list)
+            and "state" not in payload
+        ):
+            return 400, {}, ""
+
+        webhook["callback_url"] = payload["callback_url"]
+        webhook["triggers"] = payload["triggers"]
+        webhook["state"] = payload["state"]
+
+        return 200, {}, json.dumps(webhook)
+
+    endpoint = "{base}/a/{client_id}/webhooks".format(base=api_url, client_id=client_id)
+    mocked_responses.add_callback(
+        responses.POST,
+        endpoint,
+        callback=callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_webhooks(mocked_responses, api_url, client_id):
+    webhook = {
+        "application_id": "application-id",
+        "callback_url": "https://your-server.com/webhook",
+        "id": "webhook-id",
+        "state": "active",
+        "triggers": ["message.created"],
+        "version": "2.0",
+    }
+
+    def list_callback(request):
+        return 200, {}, json.dumps([webhook])
+
+    def single_callback(request):
+        webhook["id"] = get_id_from_url(request.url)
+        return 200, {}, json.dumps(webhook)
+
+    def update_callback(request):
+        try:
+            payload = json.loads(request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        if "state" in payload:
+            webhook["state"] = payload["state"]
+        webhook["id"] = get_id_from_url(request.url)
+        return 200, {}, json.dumps(webhook)
+
+    def delete_callback(request):
+        return 200, {}, json.dumps({"success": True})
+
+    def get_id_from_url(url):
+        path = URLObject(url).path
+        return path.rsplit("/", 1)[-1]
+
+    endpoint_single = re.compile(
+        "{base}/a/{client_id}/webhooks/*".format(base=api_url, client_id=client_id)
+    )
+    endpoint_list = "{base}/a/{client_id}/webhooks".format(
+        base=api_url, client_id=client_id
+    )
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_list,
+        content_type="application/json",
+        callback=list_callback,
+    )
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_single,
+        content_type="application/json",
+        callback=single_callback,
+    )
+    mocked_responses.add_callback(
+        responses.PUT,
+        endpoint_single,
+        content_type="application/json",
+        callback=update_callback,
+    )
+    mocked_responses.add_callback(
+        responses.DELETE,
+        endpoint_single,
+        content_type="application/json",
+        callback=delete_callback,
+    )
+
+
 @pytest.fixture
 def mock_resources(mocked_responses, api_url):
     resources = [
@@ -1700,6 +1800,7 @@ def mock_account_management(mocked_responses, api_url, account_id, client_id):
         "provider": "gmail",
         "organization_unit": "label",
         "billing_state": "paid",
+        "authentication_type": "password",
     }
     paid_response = json.dumps(account)
     account["billing_state"] = "cancelled"
@@ -2141,3 +2242,459 @@ def mock_clean_conversation(mocked_responses, api_url, account_id):
         content_type="application/json",
         callback=file_callback,
     )
+
+
+@pytest.fixture
+def mock_deltas_since(mocked_responses, api_url):
+    deltas = {
+        "cursor_start": "start_cursor",
+        "cursor_end": "end_cursor",
+        "deltas": [
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "given_name": "First",
+                    "surname": "Last",
+                    "id": "id-1234",
+                    "object": "contact",
+                },
+                "cursor": "contact_cursor",
+                "event": "create",
+                "id": "delta-1",
+                "object": "contact",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "content_type": "text/plain",
+                    "filename": "sample.txt",
+                    "id": "id-1234",
+                    "object": "file",
+                    "size": 123,
+                },
+                "cursor": "file_cursor",
+                "event": "create",
+                "id": "delta-2",
+                "object": "file",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "to": [{"email": "foo", "name": "bar"}],
+                    "subject": "foo",
+                    "id": "id-1234",
+                    "object": "message",
+                },
+                "cursor": "message_cursor",
+                "event": "create",
+                "id": "delta-3",
+                "object": "message",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "to": [{"email": "foo", "name": "bar"}],
+                    "subject": "foo",
+                    "id": "id-1234",
+                    "object": "draft",
+                },
+                "cursor": "draft_cursor",
+                "event": "create",
+                "id": "delta-4",
+                "object": "draft",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "subject": "Subject",
+                    "id": "id-1234",
+                    "object": "thread",
+                },
+                "cursor": "thread_cursor",
+                "event": "create",
+                "id": "delta-5",
+                "object": "thread",
+            },
+            {
+                "attributes": {
+                    "id": "id-1234",
+                    "title": "test event",
+                    "when": {"time": 1409594400, "object": "time"},
+                    "participants": [
+                        {
+                            "name": "foo",
+                            "email": "bar",
+                            "status": "noreply",
+                            "comment": "This is a comment",
+                            "phone_number": "416-000-0000",
+                        },
+                    ],
+                    "ical_uid": "id-5678",
+                    "master_event_id": "master-1234",
+                    "original_start_time": 1409592400,
+                },
+                "cursor": "event_cursor",
+                "event": "create",
+                "id": "delta-6",
+                "object": "event",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "id": "id-1234",
+                    "object": "folder",
+                    "name": "inbox",
+                    "display_name": "name",
+                },
+                "cursor": "folder_cursor",
+                "event": "create",
+                "id": "delta-7",
+                "object": "folder",
+            },
+            {
+                "attributes": {
+                    "account_id": "aid-5678",
+                    "id": "id-1234",
+                    "object": "label",
+                    "name": "inbox",
+                },
+                "cursor": "label_cursor",
+                "event": "create",
+                "id": "delta-8",
+                "object": "label",
+            },
+        ],
+    }
+
+    def callback(request):
+        return 200, {}, json.dumps(deltas)
+
+    mocked_responses.add_callback(
+        responses.GET,
+        "{base}/delta".format(base=api_url),
+        callback=callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_delta_cursor(mocked_responses, api_url):
+    def callback(request):
+        return 200, {}, json.dumps({"cursor": "cursor"})
+
+    mocked_responses.add_callback(
+        responses.POST,
+        "{base}/delta/latest_cursor".format(base=api_url),
+        callback=callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_delta_stream(mocked_responses, api_url):
+    delta = {
+        "attributes": {
+            "account_id": "aid-5678",
+            "given_name": "First",
+            "surname": "Last",
+            "id": "id-1234",
+            "object": "contact",
+        },
+        "cursor": "contact_cursor",
+        "event": "create",
+        "id": "delta-1",
+        "object": "contact",
+    }
+
+    def stream_callback(request):
+        return 200, {}, json.dumps(delta)
+
+    def longpoll_callback(request):
+        response = {
+            "cursor_start": "start_cursor",
+            "cursor_end": "end_cursor",
+            "deltas": [delta],
+        }
+        return 200, {}, json.dumps(response)
+
+    mocked_responses.add_callback(
+        responses.GET,
+        "{base}/delta/streaming".format(base=api_url),
+        callback=stream_callback,
+        content_type="application/json",
+    )
+
+    mocked_responses.add_callback(
+        responses.GET,
+        "{base}/delta/longpoll".format(base=api_url),
+        callback=longpoll_callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_outbox(mocked_responses, api_url):
+    outbox_job_status = {
+        "job_status_id": "job-status-id",
+        "status": "pending",
+        "original_data": {
+            "subject": "With Love, from Nylas",
+            "to": [{"name": "Me", "email": "test@email.com"}],
+            "body": "This email was sent using the Nylas email API. Visit https://nylas.com for details.",
+        },
+        "account_id": "account-id",
+    }
+
+    def return_job_status(request):
+        response = outbox_job_status
+        payload = json.loads(request.body)
+        if "send_at" in payload:
+            response["original_data"]["send_at"] = payload["send_at"]
+            response["original_data"]["original_send_at"] = payload["send_at"]
+            response["original_data"]["retry_limit_datetime"] = payload["send_at"]
+        if "retry_limit_datetime" in payload:
+            response["original_data"]["retry_limit_datetime"] = payload[
+                "retry_limit_datetime"
+            ]
+        return 200, {}, json.dumps(response)
+
+    def delete_callback(request):
+        return 200, {}, ""
+
+    outbox_endpoint = "{base}/v2/outbox".format(base=api_url)
+    endpoint_single = re.compile("{outbox_url}/*".format(outbox_url=outbox_endpoint))
+
+    mocked_responses.add_callback(
+        responses.POST,
+        outbox_endpoint,
+        callback=return_job_status,
+        content_type="application/json",
+    )
+
+    mocked_responses.add_callback(
+        responses.PATCH,
+        endpoint_single,
+        callback=return_job_status,
+        content_type="application/json",
+    )
+
+    mocked_responses.add_callback(
+        responses.DELETE,
+        endpoint_single,
+        callback=delete_callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_outbox_send_grid(mocked_responses, api_url):
+    send_grid_verification = {
+        "results": {"domain_verified": True, "sender_verified": True}
+    }
+
+    def return_status(request):
+        return 200, {}, json.dumps(send_grid_verification)
+
+    def delete_callback(request):
+        return 200, {}, ""
+
+    verification_url = "{base}/v2/outbox/onboard/verified_status".format(base=api_url)
+    delete_url = "{base}/v2/outbox/onboard/subuser".format(base=api_url)
+
+    mocked_responses.add_callback(
+        responses.GET,
+        verification_url,
+        callback=return_status,
+        content_type="application/json",
+    )
+
+    mocked_responses.add_callback(
+        responses.DELETE,
+        delete_url,
+        callback=delete_callback,
+        content_type="application/json",
+    )
+
+
+@pytest.fixture
+def mock_integrations(mocked_responses, client_id):
+    integration = {
+        "name": "Nylas Playground",
+        "provider": "zoom",
+        "settings": {
+            "client_id": "test_client_id",
+            "client_secret": "test_client_secret",
+        },
+        "redirect_uris": ["https://www.nylas.com"],
+        "expires_in": 12000,
+    }
+
+    def list_callback(request):
+        response = {"data": [integration], "limit": 10, "offset": 0}
+        return 200, {}, json.dumps(response)
+
+    def single_callback(request):
+        integration["provider"] = get_id_from_url(request.url)
+        response = {"data": integration}
+        return 200, {}, json.dumps(response)
+
+    def update_callback(request):
+        try:
+            payload = json.loads(request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        response = {"success": True, "data": payload}
+        return 200, {}, json.dumps(response)
+
+    def delete_callback(request):
+        return 200, {}, json.dumps({"success": True})
+
+    def get_id_from_url(url):
+        path = URLObject(url).path
+        return path.rsplit("/", 1)[-1]
+
+    endpoint_post = re.compile("https://.*nylas.com/connect/integrations")
+    endpoint_single = re.compile("https://.*nylas.com/connect/integrations/.*")
+    endpoint_list = re.compile("https://.*nylas.com/connect/integrations\?.*")
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_list,
+        content_type="application/json",
+        callback=list_callback,
+    )
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_single,
+        content_type="application/json",
+        callback=single_callback,
+    )
+    mocked_responses.add_callback(
+        responses.POST,
+        endpoint_post,
+        content_type="application/json",
+        callback=update_callback,
+    )
+    mocked_responses.add_callback(
+        responses.PATCH,
+        endpoint_single,
+        content_type="application/json",
+        callback=update_callback,
+    )
+    mocked_responses.add_callback(
+        responses.DELETE,
+        endpoint_single,
+        content_type="application/json",
+        callback=delete_callback,
+    )
+
+
+@pytest.fixture
+def mock_grants(mocked_responses, client_id):
+    grant = {
+        "id": "grant-id",
+        "provider": "zoom",
+        "grant_status": "valid",
+        "email": "email@example.com",
+        "metadata": {"isAdmin": True},
+        "scope": ["meeting:write"],
+        "user_agent": "string",
+        "ip": "string",
+        "state": "my-state",
+        "created_at": 1617817109,
+        "updated_at": 1617817109,
+    }
+
+    def list_callback(request):
+        response = {"data": [grant], "limit": 10, "offset": 0}
+        return 200, {}, json.dumps(response)
+
+    def single_callback(request):
+        response = {"data": grant}
+        return 200, {}, json.dumps(response)
+
+    def update_callback(request):
+        try:
+            payload = json.loads(request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        response = {"success": True, "data": payload}
+        return 200, {}, json.dumps(response)
+
+    def delete_callback(request):
+        return 200, {}, json.dumps({"success": True})
+
+    def on_demand_sync(request):
+        return 200, {}, json.dumps(grant)
+
+    endpoint_post = re.compile("https://.*nylas.com/connect/grants")
+    endpoint_single = re.compile("https://.*nylas.com/connect/grants/.*")
+    endpoint_list = re.compile("https://.*nylas.com/connect/grants\?.*")
+    endpoint_sync = re.compile("https://.*nylas.com/connect/grants/.*/sync.*")
+    mocked_responses.add_callback(
+        responses.POST,
+        endpoint_sync,
+        content_type="application/json",
+        callback=on_demand_sync,
+    )
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_list,
+        content_type="application/json",
+        callback=list_callback,
+    )
+    mocked_responses.add_callback(
+        responses.GET,
+        endpoint_single,
+        content_type="application/json",
+        callback=single_callback,
+    )
+    mocked_responses.add_callback(
+        responses.POST,
+        endpoint_post,
+        content_type="application/json",
+        callback=update_callback,
+    )
+    mocked_responses.add_callback(
+        responses.PATCH,
+        endpoint_single,
+        content_type="application/json",
+        callback=update_callback,
+    )
+    mocked_responses.add_callback(
+        responses.DELETE,
+        endpoint_single,
+        content_type="application/json",
+        callback=delete_callback,
+    )
+
+
+@pytest.fixture
+def mock_authentication_hosted_auth(mocked_responses, client_id):
+    api_response = {
+        "success": True,
+        "data": {
+            "url": "https://accounts.nylas.com/connect/login?id=uas-hosted-id",
+            "id": "uas-hosted-id",
+            "expires_at": 0,
+            "request": {},
+        },
+    }
+
+    def hosted_auth_response(request):
+        try:
+            payload = json.loads(request.body)
+        except ValueError:
+            return 400, {}, ""
+
+        api_response["data"]["request"] = payload
+        return 200, {}, json.dumps(api_response)
+
+    endpoint = re.compile("https://.*nylas.com/connect/auth")
+    mocked_responses.add_callback(
+        responses.POST,
+        endpoint,
+        content_type="application/json",
+        callback=hosted_auth_response,
+    )
